define(["require","ext","tools","frame-data","user-info"], function(require) {
    var AppFrame = {};
    AppFrame.CurUserInfo = require("user-info");
    AppFrame.Ext = require("ext");
    AppFrame.Tools = require("tools");
    /**
     * 静态数据
     */
    AppFrame.Data = require("frame-data");

    /**
     * 框架中的功能模块集合
     */
    AppFrame.Apps = function() {
        var AppItemsStorageName = "AppItems";

        var AppItem = function() {
                this.AppID = "";
                //应用ID
                this.AppName = "";
                //应用名称
                this.AppIcon = "";
                //应用图标
                this.AppDesc = "";
                //应用，描述
                this.Ver = "";
                //应用版本号
                this.BaseUrl = "";
                //所在路径
                this.SettingPageUrl = "";
                //设置页的地址
                this.IsShow = true;
                //是否可见
                this.IsEnable = true;
                //是否可用
                this.IsAutoUpdate = true;
                //是否自动升级
                this.IsAbelDel = true;
                //是否可以被删除
                this.Sort = 0;
                //排序号
                this.Label = "";
                //分类
                this.Able = "";
                //能力关键字
            }
            /**
             * 获取所有已经加载的功能模块集合
             */
        var GetAppItems = function() {
            var _AppItems = $api.getStorage(AppItemsStorageName);
            if (!_AppItems)
                _AppItems = {};
            return _AppItems;
        };
        /**
         * 获取某个功能模块的接口
         * 接口通过回调函数返回
         * @param {Object} AppID 模块ID
         * @param {Object} CallBack 回调函数
         */
        var GetInterface = function(AppID, CallBack) {
            var AppItems = GetAppItems();
            if (AppItems[AppID]) {
                var InterFacePath = AppItems[AppID].BaseUrl + "/interface.js";
                require([InterFacePath], function(Interface) {
                    var _interFace = Interface(AppItems[AppID]);
                    if (CallBack)
                        CallBack(_interFace);
                });
            }
        };
        /**
         * 根据ID获取某个功能模块
         * @param {Object} AppID 模块ID
         */
        var Get = function(AppID) {
            var AppItems = GetAppItems();
            return AppItems[AppID];
        };

        /**
         * 根据功能点获取某个功能模块
         * @param {Object} Able 功能点
         */
        var GetByAble = function(Able) {
            var AppItems = GetAppItems();
            for (var i in AppItems) {
                var app = data[i];
                if (app.Able.indexOf(Able) > -1) {
                    return app;
                }
            }
        };

        var SetItems = function(_AppItems) {
            $api.setStorage(AppItemsStorageName, _AppItems);
        };
        /**
         * 添加模块信息到主框架中
         * @param {Object} AppInfo
         */
        var Add = function(AppInfo) {
            var AppItems = GetAppItems();
            AppItems[AppInfo.AppID] = AppInfo;
            SetItems(AppItems);
        };
        /**
         * 移除模块
         * @param {Object} AppID
         */
        var Remove = function(AppID) {
            var AppItems = GetAppItems();
            delete AppItems[AppID];
            SetItems(AppItems);
        };
        /**
         * 清空模块列表
         */
        var Clear = function() {
            $api.rmStorage(AppItemsStorageName);
        };

        /**
         * 檢測是否需要升級
         * @param {Object} OldVer 舊的版本號
         * @param {Object} NewVer 新的版本號
         */
        var CheckAppWgtIsNeedUpdate = function(OldVer, NewVer) {
            var OldVerSub = OldVer.split('.');
            var NewVerSub = NewVer.split('.');
            if (Number(OldVerSub[0]) < Number(NewVerSub[0])) {
                return true;
            } else {
                if (Number(OldVerSub[1]) < Number(NewVerSub[1])) {
                    return true;
                } else {
                    if (Number(OldVerSub[2]) < Number(NewVerSub[2])) {
                        return true;
                    }
                }
            }
            return false;
        }

        /**
         *刪除應用模塊 
         */
        var DelApp = function(appID, CB) {

                var fs = api.require('fs');
                var App = Get(appID);
                //console.log(JSON.stringify(App));
                if (!App.IsAbelDel) {
                    alert("此应用不能被删除");
                    return;
                }
                require(["then"], function(Then) {
                    Then(function(defer) {
                        fs.rmdir({
                            path: api.fsDir + "/wgt/" + appID
                        }, function(ret, err) {
                            defer(err, ret);
                        });
                    }).fin(function(defer, err, ret) {

                        if (ret.status) {
                            Remove(appID);
                        }

                        if (CB)
                            CB(ret, err);
                    })
                })
            }
            /**
             * 下载模块
             * @param {Object} id 模块ID
             * @param {Function} progress 下载进度
             * @param {Function} NewVer 下载完后的回调
             */
        var DownLoadApp = function(id, progress, callback) {
            var url = 'http://traffic.kmlckj.com/content/package/' + id + '.zip';
            var wgtDir = api.fsDir + "/wgt";
            api.download({
                url: url,
                //savePath : api.cacheDir + "/" + id + ".zip",
                report: true,
                cache: false,
                allowResume: true,
                headers: {
                    "User-Agent": 'Android'
                }
            }, function(ret, err) {
                if (ret) {
                    if (progress) {
                        //var value = ('文件大小：' + ret.fileSize + '；下载进度：' + ret.percent + '；下载状态' + ret.state + '存储路径:' + ret.savePath);
                        progress(ret.fileSize, ret.percent, ret.state, ret.savePath);
                    }

                } else {
                    var value = err.msg;
                }
                if (ret.state == 1) {
                    var zip = api.require('zip');
                    //alert(ret.savePath);
                    zip.unarchive({
                        file: ret.savePath,
                        toPath: wgtDir
                    }, function(ret, err) {
                        if (ret.status) {
                            if (callback)
                                callback({
                                    status: 1
                                }, null);
                        } else {
                            if (callback)
                                callback({
                                    status: 0
                                }, err);
                        }
                    });
                }

            });
        }

        return {
            SetItems: SetItems,
            Add: Add,
            Remove: Remove,
            Clear: Clear,
            AppItem: AppItem,
            AppItems: GetAppItems,
            Get: Get,
            GetByAble: GetByAble,
            GetInterface: GetInterface,
            CheckAppWgtIsNeedUpdate: CheckAppWgtIsNeedUpdate,
            DownLoadApp: DownLoadApp,
            DelApp: DelApp
        }
    }();

    /**
     * 框架消息总线
     */
    AppFrame.MsgBus = function() {
        /**
         * 监听事件消息
         * @param {Object} AppID
         * @param {Object} Sign
         * @param {Object} CallBack
         */
        var On = function(Sign, CallBack) {
            api.addEventListener({
                name: Sign
            }, function(ret, err) {
                var Data = ret.value;
                CallBack(Data, err);
            });
        };
        /**
         * 发送消息
         * @param {Object} SrcAppID 当前模块ID
         * @param {Object} Sign 标记
         * @param {Object} Data 数据
         */
        var Send = function(SrcAppID, Sign, Data) {
            var data = {
                MsgID: AppFrame.Tools.uuid(8, 16),
                SrcAppID: SrcAppID,
                Sign: Sign,
                Data: Data
            };
            api.sendEvent({
                name: Sign,
                extra: data
            });
        }

        return {
            On: On,
            Send: Send
        }
    }();

    /**
     * http 操作
     */
    AppFrame.Http = {
        Get: function(url, CallBack) {
            var NetWorkStatus = AppFrame.Data.NetWorkSatus.Get();
            if (NetWorkStatus == 0) {
                CallBack({
                    Result: 0
                }, {
                    statusCode: -1,
                    msg: '请开启网络访问',
                    code: 'fail'
                });
                return;
            }
            var curUserInfo = AppFrame.CurUserInfo.Get();
            var UrlParams = AppFrame.Tools.GetUrlParams(url);
            var List = [];
            for (var key in UrlParams) {
                var up = {
                    key: key.toLocaleLowerCase(),
                    value: UrlParams[key]
                };
                List.push(up);
            }
            List = List.sort(function(o, p) {
                var a, b;
                if (typeof o === "object" && typeof p === "object" && o && p) {
                    a = o["key"];
                    b = p["key"];
                    if (a === b) {
                        return 0;
                    }
                    if (typeof a === typeof b) {
                        return a < b ? -1 : 1;
                    }
                    return typeof a < typeof b ? -1 : 1;
                }
            });
            var str = "";
            for (var i in List) {
                str += List[i].key + List[i].value;
            }
            str = AppFrame.Data.RestAPIKey + str + AppFrame.Data.SecretKey;

            var GetVK = AppFrame.Tools.SHA1(str);
            var option = {
                url: url,
                method: 'get',
                cache: false,
                timeout: 10,
                dataType: 'json',
                returnAll: false,
                headers: {
                    AppKey: AppFrame.Data.AppKey,
                    GetVK: GetVK,
                    UserId: curUserInfo.UserId + "",
                    app_val: api.appVersion,
                    appid: api.appId
                }
            };
            api.ajax(option, function(ret, err) {
                CallBack(ret, err);
            });
        },

        Post: function(url, PostData, CallBack) {
            var NetWorkStatus = AppFrame.Data.NetWorkSatus.Get();
            if (NetWorkStatus == 0) {
                CallBack({
                    Result: 0
                }, {
                    statusCode: -1,
                    msg: '请开启网络访问',
                    code: 'fail'
                });
                return;
            }

            var curUserInfo = AppFrame.CurUserInfo.Get();
            //-------计算GetVK---------
            var UrlParams = AppFrame.Tools.GetUrlParams(url);
            var GetParamList = [];
            for (var key in UrlParams) {
                var up = {
                    key: key.toLocaleLowerCase(),
                    value: UrlParams[key]
                };
                GetParamList.push(up);
            }
            GetParamList = GetParamList.sort(function(o, p) {
                var a, b;
                if (typeof o === "object" && typeof p === "object" && o && p) {
                    a = o["key"];
                    b = p["key"];
                    if (a === b) {
                        return 0;
                    }
                    if (typeof a === typeof b) {
                        return a < b ? -1 : 1;
                    }
                    return typeof a < typeof b ? -1 : 1;
                }
            });

            var GetParamVKStr = "";
            for (var i in GetParamList) {
                GetParamVKStr += GetParamList[i].key + GetParamList[i].value;
            }
            GetParamVKStr = AppFrame.Data.RestAPIKey + GetParamVKStr + AppFrame.Data.SecretKey;
            var GetVK = AppFrame.Tools.SHA1(GetParamVKStr);
            //-----------计算PostVK----------------------
            var PostParamList = [];
            for (var key in PostData) {
                var up = {
                    key: key.toLocaleLowerCase(),
                    value: PostData[key]
                };
                PostParamList.push(up);
            }
            PostParamList = PostParamList.sort(function(o, p) {
                var a, b;
                if (typeof o === "object" && typeof p === "object" && o && p) {
                    a = o["key"];
                    b = p["key"];
                    if (a === b) {
                        return 0;
                    }
                    if (typeof a === typeof b) {
                        return a < b ? -1 : 1;
                    }
                    return typeof a < typeof b ? -1 : 1;
                }
            });

            var PostParamVKStr = "";
            for (var i in PostParamList) {
                PostParamVKStr += PostParamList[i].key + PostParamList[i].value;
            }
            //console.log(PostParamVKStr);
            //alert(PostParamVKStr);
            PostParamVKStr = AppFrame.Data.RestAPIKey + PostParamVKStr + AppFrame.Data.SecretKey;

            var PostVK = AppFrame.Tools.SHA1(PostParamVKStr);

            DoPost();

            function DoPost() {
                var option = {
                    url: url,
                    method: 'post',
                    cache: false,
                    timeout: 10,
                    dataType: 'json',
                    returnAll: false,
                    headers: {
                        AppKey: AppFrame.Data.AppKey,
                        GetVK: GetVK,
                        PostVK: PostVK,
                        UserId: curUserInfo.UserId + "",
                        app_val: api.appVersion,
                        appid: api.appId
                    },
                    data: {
                        values: PostData
                    }
                };
                //console.log(JSON.stringify(option));
                api.ajax(option, function(ret, err) {
                    // alert($api.jsonToStr(err));
                    if (CallBack) {
                        CallBack(ret, err);
                    }

                });
            }

        },
    }

    window.AppFrame = AppFrame;
    return AppFrame;
})